<?php
/**
 * Copyright (C) 2024  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace Civi\Api4\Action\BigBlueButton;

/**
 * Create a meeting in Big Blue Button
 *
 * Creates a meeting in Big Blue Button. See Big Blue Button API Documentation at
 * https://docs.bigbluebutton.org/development/api/#get-post-create
 *
 * @package Civi\Api4\Action\BigBlueButton
 */
class Create extends AbstractBBBAPi {

  /**
   * Returns the API call name. E.g. create or join.
   *
   * @return string
   */
  protected function getApiCallName(): string {
    return 'create';
  }

  /**
   * Serialize this object's params into an array
   *
   * @return array
   */
  public function getParams() {
    $params = parent::getParams();
    if (isset($params['meta'])) {
      $meta = $params['meta'];
      unset($params['meta']);
      if (!is_array($meta) && strlen($meta)) {
        $meta = json_decode($meta, true);
        if (!is_array($meta)) {
          $meta = [];
        }
      }
      foreach($meta as $key => $value) {
        if (is_array($value)) {
          $value = json_decode($value);
        }
        $params['meta_'.$key] = $value;
      }
    }
    return $params;
  }

  /**
   * @param string $param
   * @return bool
   */
  public function paramExists($param) {
    if ($param == 'meta') {
      return true;
    }
    return parent::paramExists($param);
  }



  /**
   * @var string
   * @required
   */
  protected $name;

  /**
   * @var string
   * @required
   */
  protected $meetingID;

  /**
   * A welcome message
   *
   * @var string
   */
  protected $welcome = null;

  /**
   * @var string
   */
  protected $dialNumber=null;

  /**
   * @var string
   */
  protected $voiceBridge=null;

  /**
   * @var int
   */
  protected $maxParticipants=null;

  /**
   * @var string
   */
  protected $logoutURL=null;

  /**
   * @var bool
   */
  protected $record=null;

  /**
   * @var int
   */
  protected $duration=null;

  /**
   * @var bool
   */
  protected $isBreakout=null;

  /**
   * @var string
   */
  protected $parentMeetingID=null;

  /**
   * @var int
   */
  protected $sequence=null;

  /**
   * @var bool
   */
  protected $freeJoin=null;

  /**
   * @var bool
   */
  protected $breakoutRoomsPrivateChatEnabled=null;

  /**
   * @var bool
   */
  protected $breakoutRoomsRecord=null;

  /**
   * @var string
   */
  protected $meta=null;

  /**
   * @var string
   */
  protected $moderatorOnlyMessage=null;

  /**
   * @var bool
   */
  protected $autoStartRecording=null;

  /**
   * @var bool
   */
  protected $allowStartStopRecording=null;

  /**
   * @var bool
   */
  protected $webcamsOnlyForModerator=null;

  /**
   * @var string
   */
  protected $bannerText=null;

  /**
   * @var string
   */
  protected $bannerColor=null;

  /**
   * @var bool
   */
  protected $muteOnStart=null;

  /**
   * @var bool
   */
  protected $allowModsToUnmuteUsers=null;

  /**
   * @var bool
   */
  protected $lockSettingsDisableCam=null;

  /**
   * @var bool
   */
  protected $lockSettingsDisableMic=null;

  /**
   * @var bool
   */
  protected $lockSettingsDisablePrivateChat=null;

  /**
   * @var bool
   */
  protected $lockSettingsDisablePublicChat=null;

  /**
   * @var bool
   */
  protected $lockSettingsDisableNotes=null;

  /**
   * @var bool
   */
  protected $lockSettingsHideUserList=null;

  /**
   * @var bool
   */
  protected $lockSettingsLockOnJoin=null;

  /**
   * @var bool
   */
  protected $lockSettingsLockOnJoinConfigurable=null;

  /**
   * @var bool
   */
  protected $lockSettingsHideViewersCursor=null;

  /**
   * @var string
   */
  protected $guestPolicy=null;

  /**
   * @var bool
   */
  protected $meetingKeepEvents=null;

  /**
   * @var bool
   */
  protected $endWhenNoModerator=null;

  /**
   * @var int
   */
  protected $endWhenNoModeratorDelayInMinutes=null;

  /**
   * @var string
   */
  protected $meetingLayout=null;

  /**
   * @var bool
   */
  protected $learningDashboardEnabled=null;

  /**
   * @var int
   */
  protected $learningDashboardCleanupDelayInMinutes=null;

  /**
   * @var bool
   */
  protected $allowModsToEjectCameras=null;

  /**
   * @var bool
   */
  protected $allowRequestsWithoutSession=null;

  /**
   * @var int
   */
  protected $userCameraCap=null;

  /**
   * @var int
   */
  protected $meetingCameraCap=null;

  /**
   * @var int
   */
  protected $meetingExpireIfNoUserJoinedInMinutes=null;

  /**
   * @var int
   */
  protected $meetingExpireWhenLastUserLeftInMinutes=null;

  /**
   * @var string
   */
  protected $groups=null;

  /**
   * @var string
   */
  protected $logo=null;

  /**
   * @var string
   */
  protected $disabledFeatures=null;

  /**
   * @var string
   */
  protected $disabledFeaturesExclude=null;

  /**
   * @var bool
   */
  protected $preUploadedPresentationOverrideDefault=null;

  /**
   * @var bool
   */
  protected $notifyRecordingIsOn=null;

  /**
   * @var string
   */
  protected $presentationUploadExternalUrl=null;

  /**
   * @var string
   */
  protected $presentationUploadExternalDescription=null;

  /**
   * @var bool
   */
  protected $recordFullDurationMedia=null;

  /**
   * @var string
   */
  protected $preUploadedPresentation=null;

  /**
   * @var string
   */
  protected $preUploadedPresentationName=null;

  /**
   * @var bool
   */
  protected $allowPromoteGuestToModerator=null;

}
