<?php
/**
 * Copyright (C) 2024  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace Civi\BigBlueButton\ActionProvider;

use Civi\ActionProvider\Action\AbstractAction;
use Civi\ActionProvider\Parameter\ParameterBagInterface;
use Civi\ActionProvider\Parameter\Specification;
use Civi\ActionProvider\Parameter\SpecificationBag;
use Civi\Api4\Action\BigBlueButton\AbstractBBBAPi;
use CRM_Bigbluebutton_ExtensionUtil as E;

abstract class AbstractBBBAction extends AbstractAction {

  /**
   * @return \Civi\Api4\Action\BigBlueButton\AbstractBBBAPi
   */
  abstract protected function getApi4Class(): AbstractBBBAPi;

  /**
   * Run the action
   *
   * @param ParameterBagInterface $parameters
   *   The parameters to this action.
   * @param ParameterBagInterface $output
   *   The parameters this action can send back
   *
   * @return void
   */
  protected function doAction(ParameterBagInterface $parameters, ParameterBagInterface $output) {
    try {
      $api = $this->getApi4Class();
      $values = [];
      foreach($api->getParamInfo() as $name => $param) {
        if (in_array($name, ['checkPermissions', 'debug', 'version', 'chain'])) {
          continue;
        }
        if ($parameters->doesParameterExists($name)) {
          $value = $parameters->getParameter($name);
          $spec = $this->getParameterSpecification()->getSpecificationByName($name);
          if ($spec) {
            switch ($spec->getType()) {
              case 'Boolean':
                $value = (bool) $value;
                break;
            }
            $values[$name] = $value;
          }
        } elseif ($this->configuration->doesParameterExists($name)) {
          $value = $this->configuration->getParameter($name);
          $spec = $this->getConfigurationSpecification()->getSpecificationByName($name);
          if ($spec) {
            switch ($spec->getDataType()) {
              case 'Boolean':
                $value = (bool) $value;
                break;
            }
            $values[$name] = $value;
          }
        }
      }
      $api->setCheckPermissions(FALSE);
      $api->setValues($values);
      $result = $api->execute();
      foreach($this->getOutputSpecification() as $spec) {
        if (isset($result[0][$spec->getName()])) {
          $value = $result[0][$spec->getName()];
          switch ($spec->getDataType()) {
            case 'Boolean':
              $value = (strtolower($value) == 'true');
              break;
            case 'String':
              if (is_array($value)) {
                $value = json_encode($value);
              }
          }
          $output->setParameter($spec->getName(), $value);
        }
      }
    }
    catch (\Throwable $e) {
      echo $e->getMessage(); exit();
    }
  }

  /**
   * Returns the specification of the parameters of the actual action.
   *
   * @return SpecificationBag
   */
  public function getParameterSpecification(): SpecificationBag {
    $api = $this->getApi4Class();
    $bag = new SpecificationBag();
    foreach($api->getParamInfo()  as $name => $param) {
      if (in_array($name, ['checkPermissions', 'debug', 'version', 'chain', 'language'])) {
        continue;
      }
      $type = $param['type'];
      if (is_array($type)) {
        $type = reset($type);
      }
      if (empty($type)) {
        $type = 'String';
      }
      $title = $name;
      $default = $param['default'];
      $options = null;
      if (isset($param['options']) && is_array($param['options'])) {
        $options = $param['options'];
      }
      switch ($type) {
        case 'string':
          $type = 'String';
          break;
        case 'int':
          $type = 'Integer';
          break;
        case 'bool':
          $type = 'Boolean';
          $options = [
            false => E::ts('False'),
            true => E::ts('True'),
          ];
          break;

      }
      $spec = new Specification($name, $type, $title, false, $default, null, $options);
      $bag->addSpecification($spec);
    }
    return $bag;
  }

  /**
   * Returns the specification of the configuration options for the actual
   * action.
   *
   * @return SpecificationBag
   */
  public function getConfigurationSpecification() {
    return $this->getParameterSpecification();
  }

}
