<?php
/**
 * Copyright (C) 2024  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace Civi\BigBlueButton;

use CRM_Utils_System;
use GuzzleHttp\Client;

class Connector {

  /**
   * @var \Civi\BigBlueButton\Credentials
   */
  protected $credentials;

  public function __construct() {
    $this->credentials = bigbluebutton_credentials();
  }

  public function setWebhookUrl(): bool {
    if ($this->credentials->enableWebhook()) {
      $webhookUrl = $this->credentials->getWebhookUrl();
      if (empty($webhookUrl)) {
        $webhookUrl = CRM_Utils_System::url('civicrm/bigbluebutton/webhook', [], TRUE);
      }
      $parameters['callbackURL'] = $webhookUrl;
      if ($this->get('hooks/create', $parameters)) {
        return true;
      } else {
        return false;
      }
    }
    return true;
  }

  /**
   * Make a call with HTTP method: POST
   *
   * @param string $apiCallName
   * @param array $parameters
   *
   * @return Call
   */
  public function post(string $apiCallName, array $parameters): Call {
    return $this->call('POST', $apiCallName, $parameters);
  }

  /**
   * Make a call with HTTP method: GET
   *
   * @param string $apiCallName
   * @param array $parameters
   *
   * @return Call
   */
  public function get(string $apiCallName, array $parameters): Call {
    return $this->call('GET', $apiCallName, $parameters);
  }

  /**
   * @param string $method
   * @param string $apiCallName
   * @param array $parameters
   *
   * @return Call
   */
  public function call(string $method, string $apiCallName, array $parameters, bool $dryRun = false): Call {
    try {
      $url = $this->credentials->getEndPointUrl() . 'api/' . $apiCallName . '?';
      $body = [];
      if ($method == 'GET') {
        $strParamaters = $this->buildHTTPQuery($parameters);
        $url .= $strParamaters;
        $checksum = $this->generateChecksum($apiCallName, $strParamaters);
      }
      else {
        $body = $parameters;
        $checksum = $this->generateChecksum($apiCallName, '');
      }
      $url .= '&checksum=' . $checksum;
      $call = new Call($url, $method, $parameters, $checksum);
      if ($dryRun) {
        $call->setDryRun();
        return $call;
      }

      $client = new Client([
        'base_uri' => $url,
        'form_params' => $body,
        'curl' => [
          CURLOPT_RETURNTRANSFER => TRUE,
          //CURLOPT_SSL_VERIFYPEER => \Civi::settings()->get('verifySSL'),
          CURLOPT_SSL_VERIFYHOST => FALSE,
          CURLOPT_SSL_VERIFYPEER => FALSE,
        ],
      ]);
      $response = $client->request($method);
      $call->setResponse($response->getBody()->getContents());
    }
    catch (\Throwable $e) {
      $call->setErrorMessage($e->getMessage());
    }
    return $call;
  }

  protected function generateChecksum(string $apiCallName, string $parameters): string {
    return sha1($apiCallName . $parameters . $this->credentials->getSecret());
  }

  /**
   * @param mixed $array
   */
  protected function buildHTTPQuery($array): string {
    return str_replace(['%20', '!', "'", '(', ')', '*'], [
      '+',
      '%21',
      '%27',
      '%28',
      '%29',
      '%2A'
    ], http_build_query(array_filter($array), '', '&', \PHP_QUERY_RFC3986));
  }

}
